'''
   MAC0122 Princípios de Desenvolvimento de Algoritmos
 
   NÃO MODIFIQUE ESTE ARQUIVO
   
   Este arquivo contém as definições da:

       - classe Token;
       - função tokeniza();  
       - função imprima_tokens(); e
       - função tokens_para_str().

   Além das definições de constantes como as que  descrevem 
   categorias:

       NUMERO, VARIAVEL, OPERADOR, PARENTESES
'''

# Constantes
TESTE   = False

# caracteres usados em operadores
OPERADORES = "%*/+-!^="

# caracteres usados em números inteiros
DIGITOS = "0123456789"

# ponto decimal
PONTO = "."

# todos os caracteres usados em um números float
FLOATS = DIGITOS + PONTO

# caracteres usados em nomes de variáveis
LETRAS  = "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

# abre e fecha parenteses
ABRE_FECHA_PARENTESES = "()"

# Whitespace characters: space, newline, horizontal tab,
# vertical tab, form feed, carriage return
BRANCOS    = [' ', '\n', '\t', '\v', '\f', '\r']

# caractere que indica comentário
COMENTARIO = "#"

# categorias
OPERADOR   = 1 # para operadores aritméticos e atribuição
NUMERO     = 2 # para números: todos são considerados float
VARIAVEL   = 3 # para variáveis
PARENTESES = 4 # para '(' e ')

#------------------------------------------------------------
def tokeniza(exp):
    """(str) -> list (de Tokens)

    Recebe um string exp representando uma expressão e cria 
    e retorna uma lista de objetos da classe Token representando
    os itens léxicos que formam a expressão. 

    Cada item léxico é representado por um objeto Token que
    possui é formado por um atributos: item e tipo.

    O atributo item de um token faz referência a: 

        - um float: no caso do item ser um número; ou 
        - um string no caso do item ser um operador ou 
             uma variável ou um abre/fecha parenteses.

    O atributo tipo de um token faz referência à sua categoria
    (ver definição em token.py). 

        - OPERADOR;
        - NUMERO; 
        - VARIAVEL; ou 
        - PARENTESES.

    A funçao ignora tudo que esta na exp apos o caractere
    COMENTARIO (= "#").
    """
### INICIO DA REMOCAO
    n = len(exp)
    lista_tokens = []
    i = 0
    while i < n:
        token = None

        # encontramos um comentario?
        if exp[i] == COMENTARIO:
            i = n # abandona a análise
            
        # encontramos um operador?
        elif exp[i] in OPERADORES:
            # no EP6 era: token = [exp[i],OPERADOR]
            token = Token(exp[i],OPERADOR) 
            i += 1

        # encontramo um número?    
        elif exp[i] in FLOATS:
            # construa o numero
            item = ""
            while i < n and exp[i] in FLOATS:
                item += exp[i]
                i += 1

            if item.count(PONTO) > 1:
                print("AVISO: tokeniza: '%s' foi ignorado" %item)
            else:    
                item = float(item)
                # no EP6 era: token = [item,NUMERO]
                token = Token(item,NUMERO)
            
        # encontramos uma variável    
        elif exp[i] in LETRAS:
            item = ""
            while i < n and (exp[i] in LETRAS or exp[i] in DIGITOS):
                item += exp[i]
                i += 1
            # no EP6 era: token = [item,VARIAVEL]    
            token = Token(item,VARIAVEL)

        # encontramos um pega abre/fecha parenteses
        elif exp[i] in ABRE_FECHA_PARENTESES:
            # no EP6 era: token = [exp[i],PARENTESES]
            token = Token(exp[i],PARENTESES)
            i += 1

        # o que entranmos? BRANCO?
        else:
            if exp[i] not in BRANCOS:
                print("AVISO: tokeniza: símbolo '%s' foi ignorado" %exp[i])
            i += 1
             
        if token != None:    
            lista_tokens.append(token)
            
    return lista_tokens
### FIM DA REMOCAO
    print("Vixe! Ainda não fiz essa função.")
    

#---------------------------------------------------------
def imprima_tokens(tokens):
    '''(list) -> None

    Recebe uma lista de tokens em que cada objeto 
    é da classe Token e imprime a lista.

    '''
    print(tokens_para_str(tokens))
    
#------------------------------------------------------------
def tokens_para_str(tokens):
    '''(list) -> str

    Recebe uma lista tokens em que cada objeto é da classe
    Token e retorna um string representando a lista.

    Para funcionar, esta função necessita que o método
    __str__() da classe Token tenha sido implementado.
    '''
    n = len(tokens)
    if n == 0: 
        s = "[]"
    else:
        s = "["
        for token in tokens:
            s += str(token) + ", "
        s = s[:-2] + "]"
    return s    
    
 
#------------------------------------------------------------
class Token:
    ''' Classe utilizada para representar itens léxicos.

    Cada objeto têm dois atributos: item e tipo.

    O atributo item de um objeto Token é 

        - um float: no caso do item ser um número; ou 
        - um string no caso do item ser um operador ou 
             uma variável ou um abre/fecha parenteses.

    O componente tipo de um objeto token é um inteiro
    que indica a que categoria ele pertence.

        - OPERADOR   = 1;
        - NUMERO     = 2; 
        - VARIAVEL   = 3; ou 
        - PARENTESES = 4.
    '''

    #----------------------------------------------------    
    def __init__(self,item,tipo):
        '''(Token, float ou str, int) -> Token

        Construtor: cria os atributos de um objeto Token.

        '''
### INICIO DA REMOCAO
        self.item = item
        self.tipo = tipo
        return None
### FIM DA REMOCAO
        print("Vixe! Ainda não fiz esse método.")


    #----------------------------------------------------    
    def __str__(self):
        ''' (Token) -> str

        Retorna o string que print() usa para exibir
        um Token
        '''
### INICIO DA REMOCAO
        s = ''
        if   self.tipo == OPERADOR:
            s += "O('%s')" %self.item
        elif self.tipo == PARENTESES:
            s += "P('%s')" %self.item
        elif self.tipo == NUMERO:
            s += "N(%g)" %self.item
        elif self.tipo == VARIAVEL:
            s += "V('%s')" %self.item
        else:
            s += "?('%s')" %self.item
        return s
### FIM DA REMOCAO    
        print("Vixe! Ainda não fiz esse método.")

    #----------------------------------------------------
    def get_item(self):
        ''' (Token) -> str ou float

        Retorna o conteudo do atributo item do Token.

        Se o atributo tipo é OPERADOR, PARENTESES OU VARIAVEL 
        o item retornado será um string (class str).

        Se o atributo tipo é NUMERO o item retornado 
        será um float.
        '''
### INICIO DA REMOCAO
        return self.item
### FIM DA REMOCAO
        print("Vixe! Ainda não fiz esse método.")


    #----------------------------------------------------
    def get_tipo(self):
        ''' (Token) -> int

        Retorna o conteudo do atributo tipo do Token.

        Se o valor retornado é OPERADOR, PARENTESES,
        VARIAVEL ou NUMERO
        '''
### INICIO DA REMOCAO
        return self.tipo
### FIM DA REMOCAO
        print("Vixe! Ainda não fiz esse método.")

        
